@@ -1,7 +1,9 @@
+# $Id: StickyForm.pm,v 1.3 2005/10/19 15:44:21 pmh Exp $
 
 =head1 NAME
 
-HTML::StickyForm - Lightweight general-purpose HTML form generation, with sticky values
+HTML::StickyForm - Lightweight general-purpose HTML form generation,
+with sticky values
 
 =head1 SYNOPSIS
 
@@ -26,22 +28,19 @@ HTML::StickyForm - Lightweight general-purpose HTML form generation, with sticky
      "<br />Text area:",
      $f->textarea(name => 'field2', cols => 60, rows => 5, default => 'stuff'),
 
-     "<br />Single radio button:",
-     $f->radio(name => 'field3', value => 'xyz', checked => 1),
-
      "<br />Radio buttons:",
-     $f->radio_group(name => 'field4', values => [1,2,3],
+     $f->radio_group(name => 'field3', values => [1,2,3],
        labels => { 1=>'one', 2=>'two', 3=>'three' }, default => 2),
 
      "<br />Single checkbox:",
-     $f->checkbox(name => 'field5', value => 'xyz', checked => 1),
+     $f->checkbox(name => 'field4', value => 'xyz', checked => 1),
 
      "<br />Checkbox group:",
-     $f->checkbox_group(name => 'field6', values => [4,5,6],
+     $f->checkbox_group(name => 'field5', values => [4,5,6],
        labels => { 4=>'four', 5=>'five', 6=>'six' }, default => [5,6]),
 
      "<br />Password field:",
-     $f->password(name => 'field7', size => 20),
+     $f->password(name => 'field6', size => 20),
 
      '<br />",
      $f->submit(value => ' Hit me! '),
@@ -71,11 +70,12 @@ consistent, and to keep the code size down to a minimum.
 
 
 package HTML::StickyForm;
-BEGIN {
-  $HTML::StickyForm::VERSION = '0.07_02';
-}
 use strict;
-use warnings;
+use vars qw(
+  $VERSION
+);
+
+$VERSION=0.07;
 
 =head1 CLASS METHODS
 
@@ -284,11 +284,11 @@ C<method>: Defaults to C<GET>
 =cut
 
 sub form_start{
-  my($self,$args)=&_args;
-  $args->{method}='GET' unless exists $args->{method};
+  my($self,%args)=@_;
+  $args{method}='GET' unless exists $args{method};
 
   my $field='<form';
-  while(my($name,$val)=each %$args){
+  while(my($name,$val)=each %args){
     _escape($name);
     _escape($val);
     $field.=qq( $name="$val");
@@ -305,9 +305,8 @@ As form_start(), but the C<enctype> argument defaults to C<multipart/form-data>.
 =cut
 
 sub form_start_multipart{
-  my($self,$args)=&_args;
-  $args->{enctype}||='mutipart/form-data';
-  $self->form_start($args);
+  my $self=shift;
+  $self->form_start(enctype => 'mutipart/form-data',@_);
 }
 
 =item form_end()
@@ -339,15 +338,15 @@ A C<default> attribute is never created.
 =cut
 
 sub text{
-  my($self,$args)=&_args;
-  my $type=delete $args->{type} || 'text';
-  my $name=delete $args->{name};
+  my($self,%args)=@_;
+  my $type=delete $args{type} || 'text';
+  my $name=delete $args{name};
   my $value;
-  if(exists $args->{value}){
-    $value=delete $args->{value};
-    delete $args->{default};
+  if(exists $args{value}){
+    $value=delete $args{value};
+    delete $args{default};
   }else{
-    $value=delete $args->{default};
+    $value=delete $args{default};
     $value=$self->{req}->param($name) if $self->{params};
   }
 
@@ -356,7 +355,7 @@ sub text{
   _escape($value);
 
   my $field=qq(<input type="$type" name="$name" value="$value");
-  while(my($key,$val)=each %$args){
+  while(my($key,$val)=each %args){
     _escape($key);
     _escape($val);
     $field.=qq( $key="$val");
@@ -372,9 +371,8 @@ As text(), but produces an input element of type C<hidden>.
 =cut
 
 sub hidden{
-  my($self,$args)=&_args;
-  $args->{type}||='hidden';
-  $self->text($args);
+  my $self=shift;
+  $self->text(@_,type => 'hidden');
 }
 
 =item password(PAIRLIST)
@@ -384,9 +382,8 @@ As text(), but produces an input element of type C<password>.
 =cut
 
 sub password{
-  my($self,$args)=&_args;
-  $args->{type}||='password';
-  $self->text($args);
+  my $self=shift;
+  $self->text(@_,type => 'password');
 }
 
 =item textarea(PAIRLIST)
@@ -407,14 +404,14 @@ A C<default> attribute is never created.
 =cut
 
 sub textarea{
-  my($self,$args)=&_args;
-  my $name=delete $args->{name};
+  my($self,%args)=@_;
+  my $name=delete $args{name};
   my $value;
-  if(exists $args->{value}){
-    $value=delete $args->{value};
-    delete $args->{default};
+  if(exists $args{value}){
+    $value=delete $args{value};
+    delete $args{default};
   }else{
-    $value=delete $args->{default};
+    $value=delete $args{default};
     $value=$self->{req}->param($name) if $self->{params};
   }
 
@@ -422,7 +419,7 @@ sub textarea{
   _escape($value);
 
   my $field=qq(<textarea name="$name");
-  while(my($key,$val)=each %$args){
+  while(my($key,$val)=each %args){
     _escape($key);
     _escape($val);
     $field.=qq( $key="$val");
@@ -453,16 +450,15 @@ true for the C<checked> attribute to be created.
 =cut
 
 sub checkbox{
-  my($self,$args)=&_args;
-  my $type=delete $args->{type} || 'checkbox';
-  my $name=delete $args->{name};
-  my $value=delete $args->{value};
+  my($self,%args)=@_;
+  my $name=delete $args{name};
+  my $value=delete $args{value};
   my $checked;
-  if(exists $args->{checked}){
-    $checked=delete $args->{checked};
-    delete $args->{default};
+  if(exists $args{checked}){
+    $checked=delete $args{checked};
+    delete $args{default};
   }else{
-    $checked=delete $args->{default};
+    $checked=delete $args{default};
     $value='' unless defined($value);
     $checked=grep $_ eq $value,$self->{req}->param($name) if $self->{params};
   }
@@ -470,9 +466,9 @@ sub checkbox{
   _escape($name);
   _escape($value);
 
-  my $field=qq(<input type="$type" name="$name" value="$value");
+  my $field=qq(<input type="checkbox" name="$name" value="$value");
   $field.=' checked="checked"' if $checked;
-  while(my($key,$val)=each %$args){
+  while(my($key,$val)=each %args){
     _escape($key);
     _escape($val);
     $field.=qq( $key="$val");
@@ -486,10 +482,8 @@ sub checkbox{
 Generates a group of C<checkbox> type C<E<lt>inputE<gt>> elements. If called in
 list context, returns a list of elements, otherwise a single string containing
 all tags. All arguments are used directly to generate attributes in each tag,
-except for those listed below. Arguments with scalar values result in that
-value being used for each element, whereas hashref values result in the value
-keyed by the element's C<value> attribute being used.
-Unless otherwise stated, all names and values are HTML-escaped.
+except for those listed below. Unless otherwise stated, all names and values
+are HTML-escaped.
 
 C<values>: An arrayref of values.
 One element will be generated for each element, in the order supplied.
@@ -522,46 +516,38 @@ C<values_as_labels> attribute.
 =cut
 
 sub checkbox_group{
-  my($self,$args)=&_args;
-  my $type=delete $args->{type} || 'checkbox';
-  my $name=delete $args->{name};
-  my $labels=delete $args->{labels} || {};
+  my($self,%args)=@_;
+  my $type=delete $args{type} || 'checkbox';
+  my $name=delete $args{name};
+  my $labels=delete $args{labels} || {};
   my $escape_labels=1;
-  $escape_labels=delete $args->{escape_labels} if exists $args->{escape_labels};
-  my $values=delete $args->{values};
+  $escape_labels=delete $args{escape_labels} if exists $args{escape_labels};
+  my $values=delete $args{values};
   $values||=[keys %$labels];
   my $checked=[];
-  if(exists $args->{checked}){
-    $checked=delete $args->{checked};
+  if(exists $args{checked}){
+    $checked=delete $args{checked};
     $checked=[$checked] if ref($checked) ne 'ARRAY';
-    delete $args->{default};
+    delete $args{default};
   }else{
-    if(exists $args->{default}){
-      $checked=delete $args->{default};
+    if(exists $args{default}){
+      $checked=delete $args{default};
       $checked=[$checked] if ref($checked) ne 'ARRAY';
     }
     $checked=[$self->{req}->param($name)] if $self->{params};
   }
-  my %checked=map +($_,'checked'),@$checked;
-  my $br=delete $args->{linebreak} ? "<br$self->{well_formed}>" : '';
+  my %checked=map +($_,1),@$checked;
+  my $br=delete $args{linebreak} ? "<br$self->{well_formed}>" : '';
   my $v_as_l=$self->{values_as_labels};
-  if(exists $args->{values_as_labels}){
-    $v_as_l=delete $args->{values_as_labels};
+  if(exists $args{values_as_labels}){
+    $v_as_l=delete $args{values_as_labels};
   }
 
   _escape($type);
   _escape($name);
 
   my $field=qq(<input type="$type" name="$name");
-  my %per_value=(
-    checked => \%checked,
-    value => {map +($_,$_),@$values},
-  );
-  while(my($key,$val)=each %$args){
-    if($val && ref($val) eq 'HASH'){
-      $per_value{$key}=$val;
-      next;
-    }
+  while(my($key,$val)=each %args){
     _escape($key);
     _escape($val);
     $field.=qq( $key="$val");
@@ -569,14 +555,9 @@ sub checkbox_group{
 
   my @checkboxes;
   for my $value(@$values){
-    my $field=$field;
-    while(my($key,$hash)=each %per_value){
-      exists $hash->{$value}
-        or next;
-      _escape($key);
-      _escape(my $val=$hash->{$value});
-      $field.=qq( $key="$val");
-    }
+    _escape(my $evalue=$value);
+    my $field=qq($field value="$evalue");
+    $field.=' checked="checked"' if $checked{$value};
     $field.="$self->{well_formed}>";
 
     if(exists $labels->{$value}){
@@ -584,7 +565,6 @@ sub checkbox_group{
       _escape($label) if $escape_labels;
       $field.=$label;
     }elsif($v_as_l){
-      _escape(my $evalue=$value);
       $field.=$evalue;
     }
     $field.=$br;
@@ -595,18 +575,6 @@ sub checkbox_group{
   return join '',@checkboxes;
 }
 
-=item radio(PAIRLIST)
-
-As radio_group(), but setting C<type> to C<radio>.
-
-=cut
-
-sub radio{
-  my($self,$args)=&_args;
-  $args->{type}||='radio';
-  $self->checkbox($args);
-}
-
 =item radio_group(PAIRLIST)
 
 As checkbox_group(), but setting C<type> to C<radio>.
@@ -614,23 +582,19 @@ As checkbox_group(), but setting C<type> to C<radio>.
 =cut
 
 sub radio_group{
-  my($self,$args)=&_args;
-  $args->{type}||='radio';
-  $self->checkbox_group($args);
+  my $self=shift;
+
+  $self->checkbox_group(@_,type => 'radio');
 }
 
-=item select(PAIRLIST)
+=item select(%args)
 
 Generates a C<E<lt>selectE<gt>> element. All arguments are used directly to
 generate attributes in the C<E<lt>selectE<gt>> element, except for those listed below. Unless otherwise stated, all names and values are HTML-escaped.
 
-C<values>: An arrayref of values and/or option groups.
-Scalar values are used directly to create C<E<lt>optionE<gt>> elements,
-whereas arrayrefs represent option groups. The first element in an option
-group is either the group's label or a hashref holding all of the group's
-attributes, of which C<disabled> is special cased to produce the attribute
-value C<disabled> if true, and no attribute if false.
-Defaults to label keys.
+C<values>: An arrayref of values.
+One C<E<lt>optionE<gt>> element will be created inside the C<E<lt>selectE<gt>>
+element for each entry, in the supplied order.  Defaults to label keys.
 
 C<labels>: A hashref of labels.
 Each C<E<lt>optionE<gt>> tag generated will contain the
@@ -641,7 +605,7 @@ C<selected>: Unconditional status. If present, the supplied values will be
 used to decide which options to mark as selected, and C<default> and any
 sticky values will be ignored. May be a single value or arrayref.
 
-C<default>: Conditional status, ignored if C<selected> is
+C<default>: Consitional status, ignored if C<selected> is
 supplied. If the form is sticky, the sticky values will be used to decide which
 options are selected. Otherwise, the supplied values will be used.
 May be a single value or arrayref.
@@ -654,30 +618,30 @@ This is of little value, since it's the default behaviour of HTML in any case.
 =cut
 
 sub select{
-  my($self,$args)=_args(@_);
-  my $name=delete $args->{name};
-  my $multiple=delete $args->{multiple};
-  my $labels=delete $args->{labels} || {};
-  my $values=delete $args->{values} || [keys %$labels];
+  my($self,%args)=@_;
+  my $name=delete $args{name};
+  my $multiple=delete $args{multiple};
+  my $labels=delete $args{labels} || {};
+  my $values=delete $args{values} || [keys %$labels];
   my $selected;
-  if(exists $args->{selected}){
-    $selected=delete $args->{selected};
-    delete $args->{default};
+  if(exists $args{selected}){
+    $selected=delete $args{selected};
+    delete $args{default};
   }else{
-    $selected=delete $args->{default};
+    $selected=delete $args{default};
     $selected=[$self->{req}->param($name)] if $self->{params};
   }
   if(!defined $selected){ $selected=[]; }
   elsif(ref($selected) ne 'ARRAY'){ $selected=[$selected]; }
   my %selected=map +($_,1),@$selected;
   my $v_as_l=$self->{values_as_labels};
-  if(exists $args->{values_as_labels}){
-    $v_as_l=delete $args->{values_as_labels};
+  if(exists $args{values_as_labels}){
+    $v_as_l=delete $args{values_as_labels};
   }
 
   _escape($name);
   my $field=qq(<select name="$name");
-  while(my($key,$val)=each %$args){
+  while(my($key,$val)=each %args){
     _escape($key);
     _escape($val);
     $field.=qq( $key="$val");
@@ -685,14 +649,26 @@ sub select{
   $field.=' multiple="multiple"' if $multiple;
   $field.=">";
 
-  $field.=_select_options($values,\%selected,$labels,$v_as_l);
+  for my $value(@$values){
+    _escape(my $evalue=$value);
+    $field.=qq(<option value="$evalue");
+    $field.=' selected="selected"' if $selected{$value};
+    $field.=">";
+    if(exists $labels->{$value}){
+      my $label=$labels->{$value};
+      _escape($label);
+      $field.=$label;
+    }elsif($v_as_l){
+      $field.=$evalue;
+    }
+    $field.="</option>";
+  }
+
   $field.="</select>";
 
   $field;
 }
 
-
-
 =item submit(PAIRLIST)
 
 Generates an C<E<lt>inputE<gt>> of type C<submit>. All of the supplied
@@ -702,11 +678,11 @@ fields are not sticky.
 =cut
 
 sub submit{
-  my($self,$args)=_args(@_);
-  $args->{type}='submit' unless exists $args->{type};
+  my($self,%args)=@_;
+  $args{type}='submit' unless exists $args{type};
 
   my $field='<input';
-  while(my($key,$val)=each %$args){
+  while(my($key,$val)=each %args){
     _escape($key);
     _escape($val);
     $field.=qq( $key="$val");
@@ -745,68 +721,6 @@ sub _escape($){
   }
 }
 
-=item _args(@_)
-
-Work out which of the two argument passing conventions is being used, and
-return ($self,\%args). This essentially converts the public unrolled
-PAIRLIST arguments into a single hashref, as used by the internal
-interfaces.
-
-=cut
-
-sub _args{
-  my $self=shift;
-  my $args=ref($_[0]) ? {%{$_[0]}} : {@_};
-  ($self,$args);
-}
-
-=item _select_options(\@values,\%selected,\%labels,$values_as_labels)
-
-Returns an HTML fragment containing C<option> elements for the supplied
-list of option values. Values which are arrayrefs are used to represent
-option groups, wherein the zeroth element is either the group name, or
-a hashref holding the group's attributes.
-
-=cut
-
-sub _select_options{
-  my($values,$selected,$labels,$v_as_l)=@_;
-  my $field='';
-  for my $value(@$values){
-    if(ref $value){
-      # Handle option group
-      my($_group,@subvalues)=@$value;
-      my %group=ref($_group) ? %$_group : (label => $_group);
-      if(delete $group{disabled}){
-        $group{disabled}='disabled';
-      }
-      $field.=qq(<optgroup);
-      while(my($name,$value)=each %group){
-        _escape($value);
-	$field.=qq( $name="$value");
-      }
-      $field.='>';
-      $field.=_select_options(\@subvalues,$selected,$labels);
-      $field.='</optgroup>';
-    }else{
-      # Handle single option
-      _escape(my $evalue=$value);
-      $field.=qq(<option value="$evalue");
-      $field.=' selected="selected"' if $selected->{$value};
-      $field.=">";
-      if(exists $labels->{$value}){
-	my $label=$labels->{$value};
-	_escape($label);
-	$field.=$label;
-      }elsif($v_as_l){
-	$field.=$evalue;
-      }
-      $field.="</option>";
-    }
-  }
-
-  $field;
-}
 
 =back
 
@@ -821,7 +735,7 @@ sub _select_options{
 
 =head1 AUTHOR
 
-Copyright (C) Institute of Physics Publishing 2000-2011
+Copyright (C) Institute of Physics Publishing 2000-2005
 
 	Peter Haworth <pmh@edison.ioppublishing.com>
 
