@@ -1,3 +1,4 @@
+# $Id: RequestHash.pm,v 1.1 2005/10/19 13:54:20 pmh Exp $
 
 =head1 NAME
 
@@ -24,12 +25,11 @@ available, but it is not appropriate to use L<CGI> or L<Apache::Request>.
 =cut
 
 package HTML::StickyForm::RequestHash;
-BEGIN {
-  $HTML::StickyForm::RequestHash::VERSION = '0.07_02';
-}
 
 use strict;
-use warnings;
+use vars qw($VERSION);
+
+$VERSION=0.01;
 
 =head1 CLASS METHODS
 
@@ -101,7 +101,7 @@ sub param{
 
 =head1 AUTHOR
 
-Copyright (C) Institute of Physics Publishing 2005-2011
+Copyright (C) Institute of Physics Publishing 2005
 
 	Peter Haworth <pmh@edison.ioppublishing.com>
 
